;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe03e00a786d5d6476b56ed57e597cbd9 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xd876bc0cdc6ebadf58c40c546fbacc9b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x5a07edf18f14981196a77727a3ea6847 . "./code.cg2js/cg-grid-widget.fasl")
(#x02aafc5cfddb847a91e36d4f64c9073a . "./code.cg2js/cg-utility-dialog.fasl")
(#x17dde6d1dc2bed262b7256f73ea201ba . "./code.cg2js/cg-wrap-string.fasl")
(#x108ca249f016c52245873d1da70729a4 . "./code.cg2js/cg-editable-text.fasl")
(#x99e4e865dd600246999bb78e67a2b77e . "./code.cg2js/cg-base.fasl")
(#x1decaf7634136db42b91ed3bc9fc4b46 . "./code.cg2js/cg-lisp-text.fasl")
(#x86e818c48a6a19ba2bee8fe49155cd70 . "./code.cg2js/cg-intersect.fasl")
(#x58ba580d017c6bb79a47d64cabda7df6 . "./code.cg2js/cg-choose-printer.fasl")
(#xdba43ed5a38a7d800d4b0b56279bad6e . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xccb5d1de94242f73dfe17cdbfe41bdef . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x94fd57b77ea0ea3158cfb4ea450b2e63 . "./code.cg2js/cg-font-dialog.fasl")
(#x6aec388938904a4f78baf255c47b94bb . "./code.cg2js/cg-comtab.fasl")
(#x4283654eba17625d2751b36de6ac9b33 . "./code.cg2js/cg-tooltip.fasl")
(#x886db7ab0361f183d02aba376acb8438 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xdd3db644e83bae114dc36a6e0d9661d5 . "./code.cg2js/cg-get-pixmap.fasl")
(#x070fc723d626a1a323992a18402c8c20 . "./code.cg2js/cg-status-bar.fasl")
(#xca8fd13cc4797b9428b15e37f8a72ffe . "./code.cg2js/cg-tab-control.fasl")
(#xdaacd3ddb9897aeeaa9ca7d946b80818 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x6a3fe3543bfc76700d5af5785e0b5f97 . "./code.cg2js/cg-list-view.fasl")
(#x2af8df7392ff9049dac55684c747d8fe . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x3f4385457c2ee61200f330bcc5cc6f96 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xa1b6376634e2aa787f699aae6b9d7688 . "./code.cg2js/cg-project-window.fasl")
(#x2c4a58bd5738aee579dce5e33d2098c3 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x21592a4d087dd271d8ce3c1757b2f9e6 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x997300cb3e0fff30ba8ea9db9da2c6da . "./code.cg2js/cg-scaling-stream.fasl")
(#xb4e042643a2ddc372027def1c8c52036 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x80792e13fbc6b44026bd217bcc3df327 . "./code.cg2js/cg-menu.fasl")
(#xef597c4cec3aa363ee720025bf318c8e . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xc376dd0911afc7981023274fbdd45281 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd5475afc74898af2f44febd61e194930 . "./code.cg2js/cg-split-bar.fasl")
(#x0c17ae7b170d42f0aa6c5d63e36a43c5 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x86be0a8624d0f8dd36809abfdaa34072 . "./code.cg2js/cg-timer.fasl")
(#x04fe3c3797ad723d1add48e79f81b6fa . "./code.cg2js/cg-text-or-combo.fasl")
(#x5838b2a570bbfe82b8340ae932d2def9 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xa07c5ba30a516d0f02469decfc2cac1a . "./code.cg2js/cg-string-dialog.fasl")
(#x66a0cf7b5574d2f9afe43e15195cf904 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xdb218122c2cc2d16f154b547332105df . "./code.cg2js/cg-menu-tooltip.fasl")
(#xf22302aad81806f1f3def359058cd7c0 . "./code.cg2js/cg-outline.fasl")
(#x72513594df952616d03defd2b83116fa . "./code.cg2js/cg-lettered-menu.fasl")
(#x7c8eb6a290aaf340ad1dcb8abaf6230b . "./code.cg2js/cg-graphics-context.fasl")
(#xe2286b2bea4e3eb41b85238908d0bff1 . "./code.cg2js/cg-item-list.fasl")
(#x472624b1d15451d609814793d14349da . "./code.cg2js/cg-caret.fasl")
(#x69eaf24d50237d0767f0f5581667083d . "./code.cg2js/cg-printing.fasl")
(#x93bbe09b8c481c0378d02c9e3d328af4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x59dcd40d2976f3c47f6d0c06955b7282 . "./code.cg2js/cg-progress-indicator.fasl")
(#x7b7714e42e151bddb3b212d1d5206c7c . "./code.cg2js/cg-dde-utils.fasl")
(#xc3676a45bd6e0503053a251bb8230b84 . "./code.cg2js/cg-plot-widget.fasl")
(#x8ce27059dd0861504dceb0ce62faa019 . "./code.cg2js/cg-os-window.fasl")
(#x336b3cb8c02807b279665079318d862a . "./code.cg2js/cg-calendar.fasl")
(#xb72d111ceb7c88cb70c1d6c225db8663 . "./code.cg2js/cg-chart-widget.fasl")
(#x2d315d3f6256d7c35eea8d1114699799 . "./code.cg2js/cg-file-dialog.fasl")
(#xdda3ccab43b53947f3b7fa89c514f039 . "./code.cg2js/cg-nodes.fasl")
(#x9a50da28d24fbedfb78779dfb30d25c2 . "./code.cg2js/cg-lamp.fasl")
(#xade681af806d9219f4fa903c5969eadc . "./code.cg2js/cg-html-dialog.fasl")
(#x30e251059186b05cb329dcdee339a5b9 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x108fc7692e0497a61f3b8b613b2120c9 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x2f9d5d7260f7b32be204d1e7f607a77c . "./code.cg2js/cg-choice-list.fasl")
(#xdd5ea69fd8041395595d5d73b819d926 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x9fcc337f70ee40bb2c63c626ba3e5459 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x08036293ce385e49afc261a8f45bff13 . "./code.cg2js/cg-hotspot.fasl")
(#xc37874b51db1ae82fa4081eef97c4440 . "./code.cg2js/cg-property.fasl")
(#x5d98df1844aa7bf362cb7fdd0448a513 . "./code.cg2js/cg-dropping-outline.fasl")
(#x2ffe541ae762b39c573946411feca1ab . "./code.cg2js/cg-template-string.fasl")
(#x0def0c8f781b61e1fda39517ca2b55f8 . "./code.cg2js/cg-palette.fasl")
(#x274f583de9eb7475e8f6b1e3cb1a1b8b . "./code.cg2js/cg-selected-object.fasl")
(#x1a5e6afc72150ea79c65424ed16bacfb . "./code.cg2js/cg-up-down-control.fasl")
(#x27df998dcef70bc879dc81fdf4afcdcf . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x7abc01a3d5223b5f827fe672fc200294 . "./code.cg2js/cg-layout.fasl")
(#x7ac4bc5708f6ebf8c0e7bc2066717a5d . "./code.cg2js/cg-get-position.fasl")
(#x3b8cfdc56022ff8233dedba5acd0f271 . "./code.cg2js/cg-lisp-widget.fasl")
(#x281a95396f2d199efd5bb00c4febb908 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x1c6cd8edcbe34a2a88b1af835dc60710 . "./code.cg2js/cg-object-editor.fasl")
(#x885769fa1a47e7230b77fbd088219b9e . "./code.cg2js/cg-class-grid.fasl")
(#x0929550c687518e1173651b1c12260d0 . "./code.cg2js/cg-yes-no-string.fasl")
(#x44044015a3eaed89f72d43aceb38bd59 . "./code.cg2js/cg-toggling-widget.fasl")
(#x6552fa1cc098d813341451eb704a578a . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x216dc37cd27c199d4b5ab6ab2b7e76eb . "./code.cg2js/cg-button.fasl")
(#x0bbb2459312882029b32b0356be424a4 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x57a17a4eb0869d5431980db8725f7b37 . "./code.cg2js/cg-scroll-bar.fasl")
(#x61c7ddbb4c28356fef6c59a983e0aab3 . "./code.cg2js/cg-combo-box.fasl")
(#x15f6025a50a92296f95cf8a5b6f7e866 . "./code.cg2js/cg-web-browser.fasl")
(#x8529334fe2ab1cd6626c17d72a39d47d . "./code.cg2js/cg-drawable.fasl")
(#x4e2978208b4f6d4c1d0c3a3b4d8268d0 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd25dc743f8d81a8339c6dc677e0e92a1 . "./code.cg2js/cg-message-dialog.fasl")
(#xd6144c2cd898cfffe6e54dd28549b288 . "./code.cg2js/cg-text-widget.fasl")
(#x651b85dc35e9e7b6ecd3f4b8cf303da7 . "./code.cg2js/cg-static-text.fasl")
(#xb3312be35ec57c6f83a552d6c69f0efa . "./code.cg2js/cg-class-support.fasl")
(#xcf967b8504aa938e5cba37859612f56e . "./code.cg2js/cg.fasl")
(#x2c2d87845a2bef2752d47ca68f5f2e2e . "./code.cg2js/cg-directory-dialog.fasl")
(#x01bb2374e0a2bb28e54b634a9ee5b13a . "./code.cg2js/cg-clipboard.fasl")
(#x9a040d889d06017d6ed174c2adaf3214 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x3f2aea8e19e8da154c958cd6b486ff3f . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7b372311cde0dade8be126466bef21c8 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x31ce8fe0db320ac8531442de4b2c5789 . "./code.cg2js/cg-check-box.fasl")
(#x00067a4427c8f71cf5dde607ac976e43 . "./code.cg2js/cg-color-dialog.fasl")
(#x920fe865ebda6492c4abe39e225062d0 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xd099f3a4b936b961e94e495262b4c2d9 . "./code.cg2js/cg-dialog-item.fasl")
(#xea1135fd0e33219213ce7a2e8a35b61e . "./code.cg2js/cg-edit-in-place.fasl")
(#x4c1210b213fb68f6d6825325a56137b2 . "./code.cg2js/cg-tray.fasl")
(#xbb0c88926aaae0a9f43093aef4d24b01 . "./code.cg2js/cg-paren-matching.fasl")
(#x2258477e1ae87e34dc848f42da938ff3 . "./code.cg2js/cg-player.fasl")
(#x89fddfd588c63dbd3068a79515c50db0 . "./code.cg2js/cg-yes-no-list.fasl")
(#x2d59c12f8aba1053a23b96c5a5f472a6 . "./code.cg2js/cg-common-control.fasl")
(#xb23fed587df18b81d80f9c8eb9c94e64 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xbd5ef25db3ff100f1b138482bb7a8773 . "./code.cg2js/cg-os-widget.fasl")
(#xf8134478eed43fa739bda27fdf2143c5 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x6f17af8821967284e64a187caf1fc0aa . "./code.cg2js/cg-trackbar.fasl")
(#x41472f15cb3e28441fb24ae62d212df9 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x9abc598cacf29f3f9b693bcc53492044 . "./code.cg2js/cg-html-widget.fasl")
(#x746824bb6644d01ceed0aa479d6f9f26 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xa874d189ae98862b8cacf8a60c61a7b1 . "./code.cg2js/cg-acache.fasl")
(#x794e31318a96dec00cd37f63ffb69ace . "./code.cg2js/cg-shortcut-menu.fasl")
(#x9553793a54ada135dbf9eaa381f3c5de . "./code.cg2js/cg-pixmap-widget.fasl")
(#xccf23af1ec366b4e2a789993514ee789 . "./code.cg2js/cg-icon.fasl")
(#x9bc414d5ef1a8572045ed104b3f309af . "./code.cg2js/cg-pixmap.fasl")
(#x010f67e382a5cb2d863c9c2bfd6255c4 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xdfe5bd8043fe04a5107f6908dca0d063 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x3a267731bdefee17620ac589e2cfb8eb . "./code.cg2js/cg-toolbar.fasl")
(#x93f873457c9d24c8e0cad77f82deedda . "./code.cg2js/cg-radio-button.fasl")
(#x8233895a77e160434a10e3773699b9d5 . "./code.cg2js/cg-curve.fasl")
(#xb10f9876d32f09c014c8a987c400d204 . "./code.cg2js/cg-picture-widget.fasl")
(#x11a67024d5aab1534e8f3c0e214ad139 . "./code.cg2js/cg-dev.fasl")
(#xc97a7b7283c594f0258771ea821fce16 . "./code.cg2js/cg-group-box.fasl")
(#xc1d24e57176ef6e6dab9465901beb09f . "./code.cg2js/cg-fill-texture.fasl")
(#x20798dde58f225a8b7cc3aed84acd120 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xa0ba8121898eef86cd9dc5bf977f10a9 . "./code.idejs/ide-keyshort.fasl")
(#xd643906a49112ade2007cbb50f78db2f . "./code.idejs/ide-form.fasl")
(#xa1fb263fd06294faf206fdf44276e951 . "./code.idejs/ide-find-in-files.fasl")
(#xdc6939bddf4928a663fc99ff7e56cd86 . "./code.idejs/ide-class-grapher.fasl")
(#xd356dade7cbe5f2601ff561a0f9c1295 . "./code.idejs/ide-base.fasl")
(#x2fc96280cf734849b79544a09d21fb81 . "./code.idejs/ide-class-browser.fasl")
(#x060118f37b019374d6223e198c40de6b . "./code.idejs/ide-find-definitions.fasl")
(#x56c2257ad58f680be25b121f804921b6 . "./code.idejs/ide-package-list.fasl")
(#xbd75c3ebb3522909b1a06b8c14fc4d12 . "./code.idejs/ide-grapher.fasl")
(#x57962958a092007499b688cbb1096ab2 . "./code.idejs/ide-dev.fasl")
(#x5b1af85951f9e5be581b833ba7e1bdb2 . "./code.idejs/ide.fasl")
(#x6cd7975f55a7b49562da3ec6c6e714dd . "./code.idejs/ide-trace.fasl")
(#x70a4237e359ddd17dbe36da24758b554 . "./code.idejs/ide-apropos.fasl")
(#xe020d98d9844a142b4dd1e4e36bdf9c6 . "./code.idejs/ide-inspector.fasl")
(#x0c7e585f57cab9c85895cb4d78b1c092 . "./code.idejs/ide-acache.fasl")
(#x436148cf0d746d03f3831d94103c41d0 . "./code.idejs/ide-project.fasl")
(#x7f18740e1578a827e8d2873af77f29ea . "./code.idejs/ide-window-list.fasl")
(#xeb5d2e0b21648220915f6d0b6e31e4cf . "./code.idejs/ide-intro.fasl")
(#x3498db918c24acfb574355cc674b0891 . "./code.idejs/ide-clipboard.fasl")
(#x5e718e4dbd6c74a55eb91c446a506407 . "./code.idejs/ide-editor.fasl")
(#x32de1bf5b4abc76607f15fb79020fb52 . "./code.idejs/ide-process-browser.fasl")
(#xe55aa2ee811a717a8c589ad242a3f9a6 . "./code.idejs/ide-patch-dialog.fasl")
(#x04ffff54554d7e89a53edea496dc490b . "./code.idejs/ide-navigator.fasl")
(#x29c4a4cb691acde2a2fe4116aa720ac8 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
